@@ -13,9 +13,9 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-from oslo_utils import strutils
+from urllib import parse as urlparse
 
-import six.moves.urllib.parse as urlparse
+from oslo_utils import strutils
 import webob
 
 from nova.api.openstack.api_version_request \
@@ -29,6 +29,7 @@ from nova.api import validation
 import nova.conf
 from nova import exception
 from nova.i18n import _
+from nova.limit import utils as limit_utils
 from nova import objects
 from nova.policies import quota_sets as qs_policies
 from nova import quota
@@ -106,22 +107,23 @@ class QuotaSetsController(wsgi.Controller):
 
     @wsgi.Controller.api_version("2.1", MAX_PROXY_API_SUPPORT_VERSION)
     @wsgi.expected_errors(400)
+    @validation.query_schema(quota_sets.show_query, '2.0', '2.74')
     def show(self, req, id):
         return self._show(req, id, [])
 
-    @wsgi.Controller.api_version(  # noqa
-        MIN_WITHOUT_PROXY_API_SUPPORT_VERSION, '2.56')
+    @wsgi.Controller.api_version(MIN_WITHOUT_PROXY_API_SUPPORT_VERSION, '2.56')
     @wsgi.expected_errors(400)
+    @validation.query_schema(quota_sets.show_query, '2.0', '2.74')
     def show(self, req, id):  # noqa
         return self._show(req, id, FILTERED_QUOTAS_2_36)
 
     @wsgi.Controller.api_version('2.57')  # noqa
     @wsgi.expected_errors(400)
+    @validation.query_schema(quota_sets.show_query, '2.0', '2.74')
+    @validation.query_schema(quota_sets.show_query_v275, '2.75')
     def show(self, req, id):  # noqa
         return self._show(req, id, FILTERED_QUOTAS_2_57)
 
-    @validation.query_schema(quota_sets.query_schema_275, '2.75')
-    @validation.query_schema(quota_sets.query_schema, '2.0', '2.74')
     def _show(self, req, id, filtered_quotas):
         context = req.environ['nova.context']
         context.can(qs_policies.POLICY_ROOT % 'show', {'project_id': id})
@@ -135,22 +137,24 @@ class QuotaSetsController(wsgi.Controller):
 
     @wsgi.Controller.api_version("2.1", MAX_PROXY_API_SUPPORT_VERSION)
     @wsgi.expected_errors(400)
+    @validation.query_schema(quota_sets.show_query, '2.0', '2.74')
     def detail(self, req, id):
         return self._detail(req, id, [])
 
-    @wsgi.Controller.api_version(  # noqa
-        MIN_WITHOUT_PROXY_API_SUPPORT_VERSION, '2.56')
+    @wsgi.Controller.api_version(MIN_WITHOUT_PROXY_API_SUPPORT_VERSION, '2.56')
     @wsgi.expected_errors(400)
+    @validation.query_schema(quota_sets.show_query, '2.0', '2.74')
+    @validation.query_schema(quota_sets.show_query_v275, '2.75')
     def detail(self, req, id):  # noqa
         return self._detail(req, id, FILTERED_QUOTAS_2_36)
 
     @wsgi.Controller.api_version('2.57')  # noqa
     @wsgi.expected_errors(400)
+    @validation.query_schema(quota_sets.show_query, '2.0', '2.74')
+    @validation.query_schema(quota_sets.show_query_v275, '2.75')
     def detail(self, req, id):  # noqa
         return self._detail(req, id, FILTERED_QUOTAS_2_57)
 
-    @validation.query_schema(quota_sets.query_schema_275, '2.75')
-    @validation.query_schema(quota_sets.query_schema, '2.0', '2.74')
     def _detail(self, req, id, filtered_quotas):
         context = req.environ['nova.context']
         context.can(qs_policies.POLICY_ROOT % 'detail', {'project_id': id})
@@ -168,21 +172,21 @@ class QuotaSetsController(wsgi.Controller):
     def update(self, req, id, body):
         return self._update(req, id, body, [])
 
-    @wsgi.Controller.api_version(  # noqa
-        MIN_WITHOUT_PROXY_API_SUPPORT_VERSION, '2.56')
+    @wsgi.Controller.api_version(MIN_WITHOUT_PROXY_API_SUPPORT_VERSION, '2.56')
     @wsgi.expected_errors(400)
     @validation.schema(quota_sets.update_v236)
+    @validation.query_schema(quota_sets.show_query, '2.0', '2.74')
     def update(self, req, id, body):  # noqa
         return self._update(req, id, body, FILTERED_QUOTAS_2_36)
 
     @wsgi.Controller.api_version('2.57')  # noqa
     @wsgi.expected_errors(400)
     @validation.schema(quota_sets.update_v257)
+    @validation.query_schema(quota_sets.show_query, '2.0', '2.74')
+    @validation.query_schema(quota_sets.show_query_v275, '2.75')
     def update(self, req, id, body):  # noqa
         return self._update(req, id, body, FILTERED_QUOTAS_2_57)
 
-    @validation.query_schema(quota_sets.query_schema_275, '2.75')
-    @validation.query_schema(quota_sets.query_schema, '2.0', '2.74')
     def _update(self, req, id, body, filtered_quotas):
         context = req.environ['nova.context']
         context.can(qs_policies.POLICY_ROOT % 'update', {'project_id': id})
@@ -205,10 +209,16 @@ class QuotaSetsController(wsgi.Controller):
         settable_quotas = QUOTAS.get_settable_quotas(context, project_id,
                                                      user_id=user_id)
 
+        requested_quotas = body['quota_set'].items()
+        if limit_utils.use_unified_limits():
+            # NOTE(johngarbutt) currently all info comes from keystone
+            # we don't update the database.
+            requested_quotas = []
+
         # NOTE(dims): Pass #1 - In this loop for quota_set.items(), we validate
         # min/max values and bail out if any of the items in the set is bad.
         valid_quotas = {}
-        for key, value in body['quota_set'].items():
+        for key, value in requested_quotas:
             if key == 'force' or (not value and value != 0):
                 continue
             # validate whether already used and reserved exceeds the new
@@ -241,17 +251,19 @@ class QuotaSetsController(wsgi.Controller):
 
     @wsgi.Controller.api_version("2.0", MAX_PROXY_API_SUPPORT_VERSION)
     @wsgi.expected_errors(400)
+    @validation.query_schema(quota_sets.defaults_query)
     def defaults(self, req, id):
         return self._defaults(req, id, [])
 
-    @wsgi.Controller.api_version(  # noqa
-        MIN_WITHOUT_PROXY_API_SUPPORT_VERSION, '2.56')
+    @wsgi.Controller.api_version(MIN_WITHOUT_PROXY_API_SUPPORT_VERSION, '2.56')
     @wsgi.expected_errors(400)
+    @validation.query_schema(quota_sets.defaults_query)
     def defaults(self, req, id):  # noqa
         return self._defaults(req, id, FILTERED_QUOTAS_2_36)
 
     @wsgi.Controller.api_version('2.57')  # noqa
     @wsgi.expected_errors(400)
+    @validation.query_schema(quota_sets.defaults_query)
     def defaults(self, req, id):  # noqa
         return self._defaults(req, id, FILTERED_QUOTAS_2_57)
 
@@ -268,16 +280,20 @@ class QuotaSetsController(wsgi.Controller):
     # +microversions because the resource quota-set has been deleted completely
     # when returning a response.
     @wsgi.expected_errors(())
-    @validation.query_schema(quota_sets.query_schema_275, '2.75')
-    @validation.query_schema(quota_sets.query_schema, '2.0', '2.74')
+    @validation.query_schema(quota_sets.show_query_v275, '2.75')
+    @validation.query_schema(quota_sets.show_query, '2.0', '2.74')
     @wsgi.response(202)
     def delete(self, req, id):
         context = req.environ['nova.context']
         context.can(qs_policies.POLICY_ROOT % 'delete', {'project_id': id})
         params = urlparse.parse_qs(req.environ.get('QUERY_STRING', ''))
         user_id = params.get('user_id', [None])[0]
-        if user_id:
-            objects.Quotas.destroy_all_by_project_and_user(
-                context, id, user_id)
-        else:
-            objects.Quotas.destroy_all_by_project(context, id)
+
+        # NOTE(johngarbutt) with unified limits we only use keystone, not the
+        # db
+        if not limit_utils.use_unified_limits():
+            if user_id:
+                objects.Quotas.destroy_all_by_project_and_user(
+                    context, id, user_id)
+            else:
+                objects.Quotas.destroy_all_by_project(context, id)
